import { h } from 'preact'
import { useEffect, useRef, useState } from 'preact/hooks'
import { bool, func, string } from 'prop-types'
import 'regenerator-runtime/runtime'
import { PeerTubePlayer } from '@peertube/embed-api'

import Chat from '../Chat'
import Overlay from '../VideoOverlay'
import { useToggle } from '../../hooks/utility'
import { VideoWrapper, Iframe } from './styles'

const Video = ({ playing, setPlaying, src, title, org, setInfoActive }) => {
  const videoiFrame = useRef(null)
  const overlayTimeout = useRef(null)
  const videoWrapperEl = useRef(null)
  const [videoReady, setVideoReady] = useState(false)
  // const [isFullscreen, toggleFullscreen] = useToggle(false)
  // const [chatActive, setChatActive] = useState(false)
  const [overlayActive, setOverlayActiveState] = useState(true)
  const ptVideo = useRef(null)

  useEffect(() => {
    const setVideo = async () => {
      const player = new PeerTubePlayer(videoiFrame.current)
      await player.ready

      ptVideo.current = player
      player.setVolume(100)
      setVideoReady(true)
    }
    setVideo()
  }, [])

  useEffect(() => {
    const { current: player } = ptVideo

    if (!videoReady) return

    if (playing) {
      setOverlayActiveState(false)
      player.play()
    } else {
      setOverlayActiveState(true)
      player.pause()
    }
  }, [playing])

  const goFullscreen = () => {
    // toggleFullscreen()
  }

  const exitFullscreen = () => {
    // toggleFullscreen()
  }

  const handleKeyPress = keyCode => {
    if (keyCode === 32) {
      setPlaying(!playing)
    }
    // if (keyCode === 70 && !isFullscreen) {
    // console.log('goFullscreen')
    // goFullscreen()
    // }
    // if (keyCode === 70 && isFullscreen) {
    // console.log('exitFullscreen')
    // exitFullscreen()
    // }
  }

  useEffect(() => {
    window.addEventListener('keydown', ({ keyCode }) => handleKeyPress(keyCode))

    return () =>
      window.removeEventListener('keydown', ({ keyCode }) =>
        handleKeyPress(keyCode)
      )
  }, [])

  const activateOverlay = () => {
    clearTimeout(overlayTimeout.current)
    setOverlayActiveState(true)

    overlayTimeout.current = setTimeout(
      () => setOverlayActiveState(false),
      2000
    )
  }

  return (
    <VideoWrapper
      $active={overlayActive || !playing}
      onClick={() => setPlaying(!playing)}
      onMouseMove={activateOverlay}
      // ref={videoWrapperEl}
    >
      <Overlay
        active={overlayActive || !playing}
        title={title}
        org={org}
        setInfoActive={setInfoActive}
      />
      <Iframe
        sandbox="allow-same-origin allow-scripts allow-popups"
        src={`${src}?api=1&controls=false`}
        frameborder="0"
        allowfullscreen
        ref={videoiFrame}
      />
      <Chat />
    </VideoWrapper>
  )
}

Video.propTypes = {
  playing: bool,
  setPlaying: func.isRequired,
  src: string.isRequired,
  title: string.isRequired,
  org: string,
}

export default Video
